@@ -17,10 +17,21 @@
 package org.uberfire.ext.plugin.type;
 
 import org.uberfire.ext.plugin.model.PluginType;
+import org.uberfire.workbench.category.Category;
+import org.uberfire.workbench.category.Others;
 import org.uberfire.workbench.type.ResourceTypeDefinition;
 
 public class SplashPluginResourceTypeDefinition extends BasePluginResourceTypeDefinition implements ResourceTypeDefinition {
 
+    private Category category;
+
+    public SplashPluginResourceTypeDefinition() {
+    }
+
+    public SplashPluginResourceTypeDefinition(final Others category) {
+        this.category = category;
+    }
+
     @Override
     public String getShortName() {
         return "splash plugin";
@@ -35,4 +46,9 @@ public class SplashPluginResourceTypeDefinition extends BasePluginResourceTypeDe
     public String getSuffix() {
         return "/" + PluginType.SPLASH.toString().toLowerCase() + ".plugin";
     }
+
+    @Override
+    public Category getCategory() {
+        return this.category;
+    }
 }
